/*
* sound.cxx
*
* Implementation of sound classes for Win32
*
* Portable Windows Library
*
* Copyright (c) 1993-1998 Equivalence Pty. Ltd.
*
* The contents of this file are subject to the Mozilla Public License
* Version 1.0 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS"
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
* the License for the specific language governing rights and limitations
* under the License.
*
* The Original Code is Portable Windows Library.
*
* The Initial Developer of the Original Code is Equivalence Pty. Ltd.
*
* Portions are Copyright (C) 1993 Free Software Foundation, Inc.
* All Rights Reserved.
*
* Contributor(s): ______________________________________.
*
* $Revision: 7 $
* $Author: Mikes $
* $Date: 9/29/15 10:47p $
*/

#include <ptlib.h>
#include <ptlib/pprocess.h>


#define P_FORCE_STATIC_PLUGIN 1

#include <ptlib/sound.h>
#include <ptlib/msos/ptlib/sound_win32.h>

#include <math.h>

int RecordMic(); // rtichange

bool	PSoundChannelWin32::delayWriteBuffer = false;

#ifndef _WIN32_WCE
#ifdef _MSC_VER
#pragma comment(lib, "winmm.lib")
#endif
#else
#include <ptlib/wm/mmsystemx.h>
#endif


const int kDelayTime = 250;


class PSound;

PCREATE_SOUND_PLUGIN(WindowsMultimedia, PSoundChannelWin32);

#define new PNEW


///////////////////////////////////////////////////////////////////////////////

PMultiMediaFile::PMultiMediaFile()
{
	hmmio = NULL;
}


PMultiMediaFile::~PMultiMediaFile()
{
	Close();
}


PBoolean PMultiMediaFile::CreateWaveFile(const PFilePath & filename,
										 const PWaveFormat & waveFormat,
										 DWORD dataSize)
{
	if (!Open(filename, MMIO_CREATE|MMIO_WRITE))
		return false;

	MMCKINFO mmChunk;
	mmChunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
	mmChunk.cksize = 4 + // Form type
		4 + sizeof(DWORD) + waveFormat.GetSize() + // fmt chunk
		4 + sizeof(DWORD) + dataSize;              // data chunk

	// Create a RIFF chunk
	if (!CreateChunk(mmChunk, MMIO_CREATERIFF))
		return false;

	// Save the format sub-chunk
	mmChunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	mmChunk.cksize = waveFormat.GetSize();
	if (!CreateChunk(mmChunk))
		return false;

	if (!Write(waveFormat, waveFormat.GetSize()))
		return false;

	// Save the data sub-chunk
	mmChunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	mmChunk.cksize = dataSize;
	return CreateChunk(mmChunk);
}


PBoolean PMultiMediaFile::OpenWaveFile(const PFilePath & filename,
									   PWaveFormat  & waveFormat,
									   DWORD & dataSize)
{
	// Open wave file
	if (!Open(filename, MMIO_READ | MMIO_ALLOCBUF))
		return false;

	MMCKINFO mmParentChunk, mmSubChunk;
	dwLastError = MMSYSERR_NOERROR;

	// Locate a 'RIFF' chunk with a 'WAVE' form type
	mmParentChunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
	if (!Descend(MMIO_FINDRIFF, mmParentChunk))
		return false;

	// Find the format chunk
	mmSubChunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	if (!Descend(MMIO_FINDCHUNK, mmSubChunk, &mmParentChunk))
		return false;

	// Get the size of the format chunk, allocate memory for it
	if (!waveFormat.SetSize(mmSubChunk.cksize))
		return false;

	// Read the format chunk
	if (!Read(waveFormat.GetPointer(), waveFormat.GetSize()))
		return false;

	// Ascend out of the format subchunk
	Ascend(mmSubChunk);

	// Find the data subchunk
	mmSubChunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	if (!Descend(MMIO_FINDCHUNK, mmSubChunk, &mmParentChunk))
		return false;

	// Get the size of the data subchunk
	if (mmSubChunk.cksize == 0) {
		dwLastError = MMSYSERR_INVALPARAM;
		return false;
	}

	dataSize = mmSubChunk.cksize;
	return true;
}


PBoolean PMultiMediaFile::Open(const PFilePath & filename,
							   DWORD dwOpenFlags,
							   LPMMIOINFO lpmmioinfo)
{
	MMIOINFO local_mmioinfo;
	if (lpmmioinfo == NULL) {
		lpmmioinfo = &local_mmioinfo;
		memset(lpmmioinfo, 0, sizeof(local_mmioinfo));
	}

	hmmio = mmioOpen((char *)(const char *)filename, lpmmioinfo, dwOpenFlags);

	dwLastError = lpmmioinfo->wErrorRet;

	return hmmio != NULL;
}


PBoolean PMultiMediaFile::Close(UINT wFlags)
{
	if (hmmio == NULL)
		return false;

	mmioClose(hmmio, wFlags);
	hmmio = NULL;
	return true;
}


PBoolean PMultiMediaFile::Ascend(MMCKINFO & ckinfo, UINT wFlags)
{
	dwLastError = mmioAscend(hmmio, &ckinfo, wFlags);
	return dwLastError == MMSYSERR_NOERROR;
}


PBoolean PMultiMediaFile::Descend(UINT wFlags, MMCKINFO & ckinfo, LPMMCKINFO lpckParent)
{
	dwLastError = mmioDescend(hmmio, &ckinfo, lpckParent, wFlags);
	return dwLastError == MMSYSERR_NOERROR;
}


PBoolean PMultiMediaFile::Read(void * data, PINDEX len)
{
	return mmioRead(hmmio, (char *)data, len) == (int)len;
}


PBoolean PMultiMediaFile::CreateChunk(MMCKINFO & ckinfo, UINT wFlags)
{
	dwLastError = mmioCreateChunk(hmmio, &ckinfo, wFlags);
	return dwLastError == MMSYSERR_NOERROR;
}


PBoolean PMultiMediaFile::Write(const void * data, PINDEX len)
{
	return mmioWrite(hmmio, (char *)data, len) == (int)len;
}


///////////////////////////////////////////////////////////////////////////////

PWaveFormat::PWaveFormat()
{
	size = 0;
	waveFormat = NULL;
}


PWaveFormat::~PWaveFormat()
{
	if (waveFormat != NULL)
		free(waveFormat);
}


PWaveFormat::PWaveFormat(const PWaveFormat & fmt)
{
	size = fmt.size;
	waveFormat = (WAVEFORMATEX *)malloc(size);
	PAssert(waveFormat != NULL, POutOfMemory);

	memcpy(waveFormat, fmt.waveFormat, size);
}


PWaveFormat & PWaveFormat::operator=(const PWaveFormat & fmt)
{
	if (this == &fmt)
		return *this;

	if (waveFormat != NULL)
		free(waveFormat);

	size = fmt.size;
	waveFormat = (WAVEFORMATEX *)malloc(size);
	PAssert(waveFormat != NULL, POutOfMemory);

	memcpy(waveFormat, fmt.waveFormat, size);
	return *this;
}


void PWaveFormat::PrintOn(ostream & out) const
{
	if (waveFormat == NULL)
		out << "<null>";
	else {
		out << waveFormat->wFormatTag << ','
			<< waveFormat->nChannels << ','
			<< waveFormat->nSamplesPerSec << ','
			<< waveFormat->nAvgBytesPerSec << ','
			<< waveFormat->nBlockAlign << ','
			<< waveFormat->wBitsPerSample;
		if (waveFormat->cbSize > 0) {
			out << hex << setfill('0');
			const BYTE * ptr = (const BYTE *)&waveFormat[1];
			for (PINDEX i = 0; i < waveFormat->cbSize; i++)
				out << ',' << setw(2) << (unsigned)*ptr++;
			out << dec << setfill(' ');
		}
	}
}


void PWaveFormat::ReadFrom(istream &)
{
}


void PWaveFormat::SetFormat(unsigned numChannels,
							unsigned sampleRate,
							unsigned bitsPerSample)
{
	PAssert(numChannels == 1 || numChannels == 2, PInvalidParameter);
	PAssert(bitsPerSample == 8 || bitsPerSample == 16, PInvalidParameter);

	PTRACE(2, "PWaveFormat::SetFormat() - " << numChannels << ", " << sampleRate << ", " << bitsPerSample ); // rtichange

	if (waveFormat != NULL)
		free(waveFormat);

	size = sizeof(WAVEFORMATEX);
	waveFormat = (WAVEFORMATEX *)malloc(sizeof(WAVEFORMATEX));
	PAssert(waveFormat != NULL, POutOfMemory);

	waveFormat->wFormatTag = WAVE_FORMAT_PCM;
	waveFormat->nChannels = (WORD)numChannels;
	waveFormat->nSamplesPerSec = sampleRate;
	waveFormat->wBitsPerSample = (WORD)bitsPerSample;
	waveFormat->nBlockAlign = (WORD)(numChannels*bitsPerSample/8);
	waveFormat->nAvgBytesPerSec = waveFormat->nSamplesPerSec*waveFormat->nBlockAlign;
	waveFormat->cbSize = 0;
}


void PWaveFormat::SetFormat(const void * data, PINDEX size)
{
	SetSize(size);
	memcpy(waveFormat, data, size);
}


PBoolean PWaveFormat::SetSize(PINDEX sz)
{
	if (waveFormat != NULL)
		free(waveFormat);

	size = sz;
	if (sz == 0)
		waveFormat = NULL;
	else {
		if (sz < (PINDEX)sizeof(WAVEFORMATEX))
			sz = sizeof(WAVEFORMATEX);
		waveFormat = (WAVEFORMATEX *)calloc(sz, 1);
		waveFormat->cbSize = (WORD)(sz - sizeof(WAVEFORMATEX));
	}

	return waveFormat != NULL;
}


///////////////////////////////////////////////////////////////////////////////

PSound::PSound(unsigned channels,
			   unsigned samplesPerSecond,
			   unsigned bitsPerSample,
			   PINDEX   bufferSize,
			   const BYTE * buffer)
{
	encoding = 0;
	numChannels = channels;
	sampleRate = samplesPerSecond;
	sampleSize = bitsPerSample;
	SetSize(bufferSize);
	if (buffer != NULL)
		memcpy(GetPointer(), buffer, bufferSize);
}


PSound::PSound(const PFilePath & filename)
{
	encoding = 0;
	numChannels = 1;
	sampleRate = 8000;
	sampleSize = 16;
	Load(filename);
}


PSound & PSound::operator=(const PBYTEArray & data)
{
	PBYTEArray::operator=(data);
	return *this;
}


void PSound::SetFormat(unsigned channels,
					   unsigned samplesPerSecond,
					   unsigned bitsPerSample)
{
	encoding = 0;
	numChannels = channels;
	sampleRate = samplesPerSecond;
	sampleSize = bitsPerSample;
	formatInfo.SetSize(0);
}


PBoolean PSound::Load(const PFilePath & filename)
{
	// Open wave file
	PMultiMediaFile mmio;
	PWaveFormat waveFormat;
	DWORD dataSize;
	if (!mmio.OpenWaveFile(filename, waveFormat, dataSize)) {
		dwLastError = mmio.GetLastError();
		return false;
	}

	encoding = waveFormat->wFormatTag;
	numChannels = waveFormat->nChannels;
	sampleRate = waveFormat->nSamplesPerSec;
	sampleSize = waveFormat->wBitsPerSample;

	if (encoding != 0) {
		PINDEX formatSize = waveFormat->cbSize + sizeof(WAVEFORMATEX);
		memcpy(formatInfo.GetPointer(formatSize), waveFormat, formatSize);
	}

	// Allocate and lock memory for the waveform data.
	if (!SetSize(dataSize)) {
		dwLastError = MMSYSERR_NOMEM;
		return false;
	}

	// Read the waveform data subchunk
	if (!mmio.Read(GetPointer(), GetSize())) {
		dwLastError = mmio.GetLastError();
		return false;
	}

	return true;
}


PBoolean PSound::Save(const PFilePath & filename)
{
	PWaveFormat waveFormat;
	if (encoding == 0)
		waveFormat.SetFormat(numChannels, sampleRate, sampleSize);
	else {
		waveFormat.SetSize(GetFormatInfoSize());
		memcpy(waveFormat.GetPointer(), GetFormatInfoData(), GetFormatInfoSize());
	}

	// Open wave file
	PMultiMediaFile mmio;
	if (!mmio.CreateWaveFile(filename, waveFormat, GetSize())) {
		dwLastError = mmio.GetLastError();
		return false;
	}

	if (!mmio.Write(GetPointer(), GetSize())) {
		dwLastError = mmio.GetLastError();
		return false;
	}

	return true;
}


PBoolean PSound::Play()
{
	PSoundChannel channel(PSoundChannel::GetDefaultDevice(PSoundChannel::Player),
		PSoundChannel::Player);
	if (!channel.IsOpen())
		return false;

	return channel.PlaySound(*this, true);
}

PBoolean PSound::Play(const PString & device)
{

	PSoundChannel channel(device,
		PSoundChannel::Player);
	if (!channel.IsOpen())
		return false;

	return channel.PlaySound(*this, true);
}

PBoolean PSound::PlayFile(const PFilePath & file, PBoolean wait)
{
	PVarString filename = file;
	return ::PlaySound(filename, NULL, SND_FILENAME|(wait ? SND_SYNC : SND_ASYNC));
}


///////////////////////////////////////////////////////////////////////////////

PWaveBuffer::PWaveBuffer(PINDEX sz)
: PBYTEArray(sz)
{
	hWaveOut = NULL;
	hWaveIn = NULL;
	header.dwFlags = WHDR_DONE;
}


PWaveBuffer::~PWaveBuffer()
{
	Release();
}


PWaveBuffer & PWaveBuffer::operator=(const PSound & sound)
{
	PBYTEArray::operator=(sound);
	return *this;
}


void PWaveBuffer::PrepareCommon(PINDEX count)
{
	Release();

	memset(&header, 0, sizeof(header));
	header.lpData = (char *)GetPointer();
	header.dwBufferLength = count;
	//header.dwUser = (DWORD)this;  // NOT USED
}


DWORD PWaveBuffer::Prepare(HWAVEOUT hOut, PINDEX & count)
{
	// Set up WAVEHDR structure and prepare it to be written to wave device
	if (count > GetSize())
		count = GetSize();

	PrepareCommon(count);
	hWaveOut = hOut;
	return waveOutPrepareHeader(hWaveOut, &header, sizeof(header));
}


DWORD PWaveBuffer::Prepare(HWAVEIN hIn)
{
	// Set up WAVEHDR structure and prepare it to be read from wave device
	PrepareCommon(GetSize());
	hWaveIn = hIn;
	return waveInPrepareHeader(hWaveIn, &header, sizeof(header));
}


DWORD PWaveBuffer::Release()
{
	DWORD err = MMSYSERR_NOERROR;

	// There seems to be some pathalogical cases where on an Abort() call the buffers
	// still are "in use", even though waveOutReset() was called. So wait until the
	// sound driver has finished with the buffer before releasing it.

	if (hWaveOut != NULL) {
		if ((err = waveOutUnprepareHeader(hWaveOut, &header, sizeof(header))) == WAVERR_STILLPLAYING)
			return err;
		hWaveOut = NULL;
	}

	if (hWaveIn != NULL) {
		if ((err = waveInUnprepareHeader(hWaveIn, &header, sizeof(header))) == WAVERR_STILLPLAYING)
			return err;
		hWaveIn = NULL;
	}

	header.dwFlags |= WHDR_DONE;
	return err;
}


///////////////////////////////////////////////////////////////////////////////

PSoundChannelWin32::PSoundChannelWin32()
: hWaveIn(NULL)
, hWaveOut(NULL)
, hMixer(NULL)
, hEventDone(CreateEvent(NULL, false, false, NULL))
, opened(false)
, m_reopened(false)
, bufferIndex(0)
, bufferByteOffset(P_MAX_INDEX)
{
#ifdef WAVE_START_DEBUG // rtichange...
	dwStartTickCount = 0;
	iTraceCount = 11;
#endif // ...rtichange
	// rtitodo - when trying to fix the rx buffering issue, I had attempted to set the format
	// to 8 bits and added a 3rd buffer - but I don't believe that code made a difference, but I had 
	// left it in place.  Now I'm reverting the code but making a comment here in case I run into the 
	// rx buffer issue again.  Remove this code once we know the buffer issue is solid
	// waveFormat.SetFormat(1, 8000, 8); // rtichange rtitodo
	// SetBuffers(32768, 3); // rtichange rtitodo
	waveFormat.SetFormat(1, 8000, 16);
	SetBuffers(1024/*32768*/, 2); // rtichange rtitodo - what's the buffer even for?  why so large
}

PSoundChannelWin32::~PSoundChannelWin32()
{
	Close();

	if (hEventDone != NULL)
		CloseHandle(hEventDone);
}

const char * PSoundChannelWin32::GetDriverName()
{
	return PPlugin_PSoundChannel_WindowsMultimedia::ServiceName();
}

PString PSoundChannelWin32::GetName() const
{
	return PConstString("WindowsMultimedia") + PPluginServiceDescriptor::SeparatorChar + deviceName;
}

static bool GetWaveOutDeviceName(UINT id, PString & name)
{
	if (id == WAVE_MAPPER) {
		name = "Default";
		return true;
	}

	WAVEOUTCAPS caps;
	if (waveOutGetDevCaps(id, &caps, sizeof(caps)) != 0)
		return false;

	name = PString(caps.szPname).Trim();
	return true;
}

static bool GetWaveInDeviceName(UINT id, PString & name)
{
	if (id == WAVE_MAPPER) {
		name = "Default";
		return true;
	}

	WAVEINCAPS caps;
	if (waveInGetDevCaps(id, &caps, sizeof(caps)) != 0)
		return false;

	name = PString(caps.szPname).Trim();
	return true;
}

PString PSoundChannelWin32::GetDefaultDevice(Directions)
{
	return "Default";
}

PStringArray PSoundChannelWin32::GetDeviceNames(Directions dir)
{
	PStringArray devices;

	UINT numDevs;
	UINT id = WAVE_MAPPER;

	switch (dir) {
case Player :
	numDevs = waveOutGetNumDevs();
	do {
		PCaselessString dev;
		if (GetWaveOutDeviceName(id, dev))
			devices.AppendString(dev);
	} while (++id < numDevs);
	break;

case Recorder :
	numDevs = waveInGetNumDevs();
	do {
		PCaselessString dev;
		if (GetWaveInDeviceName(id, dev))
			devices.AppendString(dev);
	} while (++id < numDevs);
	break;
case Closed:
	break;
	}

	return devices;
}

PBoolean PSoundChannelWin32::GetDeviceID(const PString & device, Directions dir, unsigned& id)
{
	PINDEX offset = device.Find(PPluginServiceDescriptor::SeparatorChar);
	if (offset == P_MAX_INDEX)
		offset = 0;
	else
		offset++;

	if (device[offset] == '#') {
		id = device.Mid(offset+1).AsUnsigned();
		switch (dir) {
case Player :
	if (id < waveOutGetNumDevs())
		GetWaveOutDeviceName(id, deviceName);
	break;

case Recorder :
	if (id < waveInGetNumDevs())
		GetWaveInDeviceName(id, deviceName);
	break;
case Closed :
	break;
		}
	}
	else {
		id = WAVE_MAPPER;
		UINT numDevs;
		switch (dir) {
case Player :
	numDevs = waveOutGetNumDevs();
	do {
		PCaselessString str;
		if (GetWaveOutDeviceName(id, str) && str == device.Mid(offset)) {
			deviceName = str;
			break;
		}
	} while (++id < numDevs);
	break;

case Recorder :
	numDevs = waveInGetNumDevs();
	do {
		PCaselessString str;
		if (GetWaveInDeviceName(id, str) && str == device.Mid(offset)) {
			deviceName = str;
			break;
		}
	} while (++id < numDevs);
	break;

case Closed :
	break;
		}
	}

	if (deviceName.IsEmpty())
		return SetErrorValues(NotFound, MMSYSERR_BADDEVICEID|PWIN32ErrorFlag);

	return true;
}

bool PSoundChannelWin32::Open(const Params & params)
{
	Close();
	unsigned id = 0;

	PTRACE(1, "PSoundChannelWin32::Open() - " << params.m_channels << ", " << params.m_sampleRate << ", " << params.m_bitsPerSample ); // rtichange

	if( !GetDeviceID(params.m_device, params.m_direction, id) )
		return false;

	waveFormat.SetFormat(params.m_channels, params.m_sampleRate, params.m_bitsPerSample);
	SetBuffers(params.m_bufferSize, params.m_bufferCount);

	activeDirection = params.m_direction;
	return OpenDevice(id);
}

PBoolean PSoundChannelWin32::Open(const PString & device,
								  Directions dir,
								  const PWaveFormat& format)
{
	Close();
	unsigned id = 0;

	PTRACE(2, "PSoundChannelWin32::Open()" ); // rtichange

	if( !GetDeviceID(device, dir, id) )
		return false;

	waveFormat = format;

	activeDirection = dir;
	return OpenDevice(id);
}

PBoolean PSoundChannelWin32::OpenDevice(P_INT_PTR id)
{
	Close();

	PWaitAndSignal mutex(bufferMutex);

	bufferByteOffset = P_MAX_INDEX;
	bufferIndex = 0;

	WAVEFORMATEX* format = (WAVEFORMATEX*) waveFormat;

#ifndef _WIN32_WCE // rtichange
	UINT mixerId = UINT_MAX;
#endif // rtichange
	MIXERLINE line;

	MMRESULT osError = MMSYSERR_BADDEVICEID;
	switch (activeDirection) {
	case Player :
		// rtichange PTRACE(4, "WinSnd\twaveOutOpen, id=" << id);
		PTRACE(2, "PSoundChannelWin32::OpenDevice() - Player!"); // rtichange
		osError = waveOutOpen(&hWaveOut, (UINT)id, format, (DWORD_PTR)hEventDone, 0, CALLBACK_EVENT);
		if (osError == MMSYSERR_NOERROR) {

// rtichange... (not using mixer for volume control - callback into the app)
#ifndef _WIN32_WCE
		if (mixerGetID((HMIXEROBJ)hWaveOut, &mixerId, MIXER_OBJECTF_HWAVEOUT) == MMSYSERR_NOERROR)
			line.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT;
#endif // rtichange
		}
		break;

	case Recorder :
		 // rtichange PTRACE(4, "WinSnd\twaveInOpen, id=" << id);
		PTRACE(2, "PSoundChannelWin32::OpenDevice() - Recorder!"); // rtichange
		osError = waveInOpen(&hWaveIn, (UINT)id, format, (DWORD_PTR)hEventDone, 0, CALLBACK_EVENT);
		if (osError == MMSYSERR_NOERROR) {

// rtichange... (mixerGetID() fails)
#ifdef _WIN32_WCE
			PTRACE(2, "PSoundChannelWin32::OpenDevice() - Recorder! - waveInOpen() returned NOERROR");
			mixerOpen(&hMixer, (UINT)hWaveIn, NULL, NULL, MIXER_OBJECTF_HWAVEIN);
			PTRACE(2, "PSoundChannelWin32::OpenDevice() - Recorder! - mixerOpen() returned " << hMixer);
			line.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE; // rtitodo MIXERLINE_COMPONENTTYPE_DST_WAVEIN;
#else // ...rtichange
			if (mixerGetID((HMIXEROBJ)hWaveIn, &mixerId, MIXER_OBJECTF_HWAVEIN) == MMSYSERR_NOERROR)
				line.dwComponentType = MIXERLINE_COMPONENTTYPE_DST_WAVEIN;
#endif // rtichange
		}
		break;

	case Closed:
		break;
	}

	if (osError != MMSYSERR_NOERROR)
		return SetErrorValues(NotFound, osError|PWIN32ErrorFlag);

	opened = true;
	os_handle = id;

#ifndef _WIN32_WCE // rtichange
	if (mixerId == UINT_MAX) {
		PTRACE(2, "WinSnd\tNo mixer available");
		return true; // Still return true as have actual device
	}

	if ((osError = mixerOpen(&hMixer, mixerId, 0, 0, MIXER_OBJECTF_MIXER)) != MMSYSERR_NOERROR) {
		PTRACE(2, "WinSnd\tFailed to open mixer, error=" << osError);
		return true; // Still return true as have actual device
	}
	line.cbStruct = sizeof(line);
	if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line, /* rtichange MIXER_OBJECTF_HMIXER | */ MIXER_GETLINEINFOF_COMPONENTTYPE)) != MMSYSERR_NOERROR) {
		PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
	}
#else // rtichange...
	if (hMixer == NULL) { // should be null for volume / player control
		return true; // Still return true as have actual device
	}
	line.cbStruct = sizeof(line);
	if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line,
		MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE)) != MMSYSERR_NOERROR) {
			PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
	}
#endif // rtichange

	else {
		bool haveControl = true;
		MIXERLINECONTROLS controls;

		if ((activeDirection == Recorder) && !PProcess::IsOSVersion(6,0,0)) { //5=XP/win2003
			/* There is no "master" for the recording side, so need to find the
			single selected input or at least individual microphone input
			No need to do all of these on Vista/Win7/Win2008 as there is
			a "master" for every input on those OS */

			DWORD iConn = line.cConnections; //should save first

			//1)Attempt to find currently selected input
			MIXERCONTROL muxControl;
			muxControl.cbStruct = sizeof(muxControl);
			controls.cbStruct = sizeof(controls);
			controls.dwLineID = line.dwLineID;
			controls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUX;
			controls.cControls = 1;
			controls.pamxctrl = &muxControl;
			controls.cbmxctrl = muxControl.cbStruct;
			if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
				MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) == MMSYSERR_NOERROR) {
					MIXERCONTROLDETAILS mxcd;
					mxcd.cbStruct = sizeof(mxcd);
					mxcd.dwControlID = muxControl.dwControlID;
					mxcd.cChannels = 1;
					mxcd.cMultipleItems = muxControl.cMultipleItems;
					mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_LISTTEXT);
					std::vector<MIXERCONTROLDETAILS_LISTTEXT> mxcdlt(mxcd.cChannels * mxcd.cMultipleItems); 
					mxcd.paDetails = &mxcdlt[0];
					// Get the control list text items (input lines id's)
					if ((osError = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
						MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_LISTTEXT)) == MMSYSERR_NOERROR) {
							mxcd.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
							std::vector<MIXERCONTROLDETAILS_BOOLEAN> mxcdb(mxcd.cChannels * mxcd.cMultipleItems);
							mxcd.paDetails = &mxcdb[0];
							// Get the control values for line states (selected/not selected)
							if ((osError = mixerGetControlDetails((HMIXEROBJ)hMixer, &mxcd,
								MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE)) == MMSYSERR_NOERROR) {
									DWORD iMult = mxcd.cMultipleItems;
									while (iMult > 0) {
										line.dwLineID = mxcdlt[--iMult].dwParam1;
										if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line,
											MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_LINEID)) != MMSYSERR_NOERROR)
											PTRACE(2, "WinSnd\tRecorder: Failed to get mixer info, error=" << osError);
										else {
											if (mxcdb[iMult].fValue == 1) {
												PTRACE(2, "WinSnd\tRecorder: Found selected input device: " << line.szName);
												break;
											}

											if (iMult == 0) {
												haveControl = false;
												PTRACE(2, "WinSnd\tRecorder: Failed to find selected input device");
											}
										}
									}
							}
							else {
								haveControl = false;
								PTRACE(2, "WinSnd\tRecorder: failed to get info about states of input lines, error=" << osError);
							}
					}
					else {
						haveControl = false;
						PTRACE(2, "WinSnd\tRecorder: failed to get info about input lines, error=" << osError);
					}
			}
			else {
				haveControl = false;
				PTRACE(2, "WinSnd\tRecorder: failed to get mixer line mux control, error=" << osError);
			}

			//2) Attempt to find microphone input if not seccessfull with selected input
			if (!haveControl) {
				haveControl = true;
				PTRACE(2, "WinSnd\tRecorder: trying to find microphone now...");
				while (iConn > 0) {
					line.dwSource = --iConn;
					if ((osError = mixerGetLineInfo((HMIXEROBJ)hMixer, &line, MIXER_GETLINEINFOF_SOURCE)) != MMSYSERR_NOERROR)
						PTRACE(2, "WinSnd\tFailed to get mixer info, error=" << osError);
					else if (line.dwComponentType == MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE) {
						PTRACE(5, "WinSnd\tFfound microphone, source=" << iConn);
						break;
					}

					if (iConn == 0) {
						PTRACE(2, "WinSnd\tFailed to find microphone info");
						haveControl = false;
					}
				}
			}
		}

		if (haveControl) {
			volumeControl.cbStruct = sizeof(volumeControl);

			controls.cbStruct = sizeof(controls);
			controls.dwLineID = line.dwLineID;
			controls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
			controls.cControls = 1;
			controls.pamxctrl = &volumeControl;
			controls.cbmxctrl = volumeControl.cbStruct;

			if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
				MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) == MMSYSERR_NOERROR) {
					muteControl.cbStruct = sizeof(muteControl);
					controls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
					controls.pamxctrl = &muteControl;
					controls.cbmxctrl = muteControl.cbStruct;
					if ((osError = mixerGetLineControls((HMIXEROBJ)hMixer, &controls,
						MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE)) != MMSYSERR_NOERROR) {
							PTRACE(2, "WinSnd\tFailed to get mixer line mute control ("
								<< (activeDirection == Recorder ? "Recorder" : "Player") << "), error=" << osError);
					}
					return true;
			}
			PTRACE(2, "WinSnd\tFailed to get mixer line control, error=" << osError);
		}
	}

	mixerClose(hMixer);
	hMixer = NULL;
	return true; // Still return true as have actual device
}

// the following code was based on the 2.6.0 opal code and was working, kept here in case the above 2.6.1 code fails.
// remove when the above code is proven to work solidly.
// #define RTI_WORKING_VERSION 1
#ifdef RTI_WORKING_VERSION
PBoolean PSoundChannelWin32::OpenDevice(P_INT_PTR id)
{
	PTRACE(1, "PSoundChannelWin32::OpenDevice() - " << id ); // rtichange

	Close();

	PWaitAndSignal mutex(bufferMutex);

	bufferByteOffset = P_MAX_INDEX;
	bufferIndex = 0;

	WAVEFORMATEX* format = (WAVEFORMATEX*) waveFormat;

	MIXERLINE line;

	MMRESULT osError = MMSYSERR_BADDEVICEID;

	switch (activeDirection) {

		case Player :
			PTRACE(1, "PSoundChannelWin32::OpenDevice() - Player!");

			osError = waveOutOpen(&hWaveOut, (UINT)id, format, (DWORD_PTR)hEventDone, 0, CALLBACK_EVENT);
			if (osError == MMSYSERR_NOERROR) {

				PTRACE(1, "PSoundChannelWin32::OpenDevice() - Player! - waveOutOpen() returned NOERROR");

				mixerOpen(&hMixer, (UINT)hWaveOut, NULL, NULL, MIXER_OBJECTF_HWAVEOUT);
				line.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT;
			}
			break;

		case Recorder :
			PTRACE(1, "PSoundChannelWin32::OpenDevice() - Recorder!");

			osError = waveInOpen(&hWaveIn, id, format, (DWORD)hEventDone, 0, CALLBACK_EVENT);

			if (osError == MMSYSERR_NOERROR) {

				PTRACE(1, "PSoundChannelWin32::OpenDevice() - Recorder! - waveInOpen() returned NOERROR");
				mixerOpen(&hMixer, (UINT)hWaveIn, NULL, NULL, MIXER_OBJECTF_HWAVEIN);

				PTRACE(1, "PSoundChannelWin32::OpenDevice() - Recorder! - mixerOpen() returned " << hMixer);
				line.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE; // rtitodo MIXERLINE_COMPONENTTYPE_DST_WAVEIN;

			} else {
				PTRACE(1, "PSoundChannelWin32::OpenDevice() - waveInOpen() returned error! - " << osError );
			}

			break;
		case Closed:
			break;
	}

	if (osError != MMSYSERR_NOERROR) {
		PTRACE(1, "PSoundChannelWin32::OpenDevice() - ERRORS AND RETURNING!!!!!!!!!!");
		return SetErrorValues(NotFound, osError|PWIN32ErrorFlag);
	}

	if (hMixer != NULL) {
		line.cbStruct = sizeof(line);

		PTRACE(1, "PSoundChannelWin32::OpenDevice() - calling mixerGetLineInfo()");

		DWORD dwMixerRet = mixerGetLineInfo((HMIXEROBJ)hMixer, &line, /*MIXER_OBJECTF_HMIXER | */ MIXER_GETLINEINFOF_COMPONENTTYPE);
		if ( dwMixerRet != MMSYSERR_NOERROR) {
			// RETAILMSG(1,(L"ERROR: mixerGetLineInfo() returned %ld\r\n", dwMixerRet ));

			PTRACE(1, "PSoundChannelWin32::OpenDevice() - mixerGetLineInfo() returned error! - " << dwMixerRet );

			mixerClose(hMixer);
			hMixer = NULL;
		}
		else {

			PTRACE(1, "PSoundChannelWin32::OpenDevice() - mixerGetLineInfo() returned success" );

			volumeControl.cbStruct = sizeof(volumeControl);

			MIXERLINECONTROLS controls;
			controls.cbStruct = sizeof(controls);
			controls.dwLineID = line.dwLineID&0xffff;
			controls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
			controls.cControls = 1;
			controls.pamxctrl = &volumeControl;
			controls.cbmxctrl = volumeControl.cbStruct;

			PTRACE(1, "PSoundChannelWin32::OpenDevice() - calling mixerGetLineControls()");

			MMRESULT result = mixerGetLineControls((HMIXEROBJ)hMixer, &controls, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE);

			if ( result != MMSYSERR_NOERROR) {
				PTRACE(1, "PSoundChannelWin32::OpenDevice() - mixerGetLineControls() returned error!! - %ld" << result );
				mixerClose(hMixer);
				hMixer = NULL;
			} else {

				PTRACE(1, "PSoundChannelWin32::OpenDevice() - mixerGetLineControls() returned success!!" );
			}
		}
	}

	opened = true;
	os_handle = id;
	return true;
}
#endif

PBoolean PSoundChannelWin32::IsOpen() const
{ 
	return opened ? true : false;
}

PBoolean PSoundChannelWin32::SetFormat(unsigned numChannels,
									   unsigned sampleRate,
									   unsigned bitsPerSample)
{
	Abort();

	waveFormat.SetFormat(numChannels, sampleRate, bitsPerSample);

	return OpenDevice(os_handle);
}

PBoolean PSoundChannelWin32::SetFormat(const PWaveFormat & format)
{
	Abort();

	waveFormat = format;

	return OpenDevice(os_handle);
}

unsigned PSoundChannelWin32::GetChannels() const
{
	return waveFormat->nChannels;
}

unsigned PSoundChannelWin32::GetSampleRate() const
{
	return waveFormat->nSamplesPerSec;
}

unsigned PSoundChannelWin32::GetSampleSize() const
{
	return waveFormat->wBitsPerSample;
}

PBoolean PSoundChannelWin32::Close()
{
	if (!IsOpen())
		return SetErrorValues(NotOpen, EBADF);

	PTRACE(1, "PSoundChannelWin32::Close() - (and was open!)"); // rtichange

	PWaitAndSignal mutex(bufferMutex);

	Abort();

	if (hWaveOut != NULL) {
		while (waveOutClose(hWaveOut) == WAVERR_STILLPLAYING)
			waveOutReset(hWaveOut);
		hWaveOut = NULL;
	}

	if (hWaveIn != NULL) {
		while (waveInClose(hWaveIn) == WAVERR_STILLPLAYING)
			waveInReset(hWaveIn);
		hWaveIn = NULL;
	}

	if (hMixer != NULL) {
		mixerClose(hMixer);
		hMixer = NULL;
	}

	opened = false;
	os_handle = -1;
	return true;
}

PBoolean PSoundChannelWin32::SetBuffers(PINDEX size, PINDEX count)
{
	if (count == buffers.GetSize() && size == buffers[0].GetSize())
		return true;

	Abort();

	PAssert(size > 0 && count > 0, PInvalidParameter);

	PTRACE(2, "PSoundChannelWin32::SetBuffers() - Setting sounds buffers to " << count << " x " << size); // rtichange

	PBoolean ok = true;

	PWaitAndSignal mutex(bufferMutex);

	if (!buffers.SetSize(count))
		ok = false;
	else {
		for (PINDEX i = 0; i < count; i++) {
			if (buffers.GetAt(i) == NULL)
				buffers.SetAt(i, new PWaveBuffer(size));
			if (!buffers[i].SetSize(size))
				ok = false;
		}
	}

	bufferByteOffset = P_MAX_INDEX;
	bufferIndex = 0;

	return ok;
}

PBoolean PSoundChannelWin32::GetBuffers(PINDEX & size, PINDEX & count)
{
	PWaitAndSignal mutex(bufferMutex);

	count = buffers.GetSize();

	if (count == 0)
		size = 0;
	else
		size = buffers[0].GetSize();

	return true;
}

int PSoundChannelWin32::WaitEvent(ErrorGroup group)
{
	// PTRACE(2, "PSoundChannelWin32::WaitEvent()"); rtichange

	switch (WaitForSingleObject(hEventDone, std::max((int)buffers[0].GetSize(), 1000))) {
case WAIT_OBJECT_0 :
	PTRACE_IF(3, m_reopened, "WinSnd\tRecovered from stuck device");
	m_reopened = false;
	return 1;

case WAIT_TIMEOUT :
	if (!IsOpen())
		return false;

	if (m_reopened) {
		PTRACE(2, "WinSnd\tCould not recover from stuck device");
		return SetErrorValues(Timeout, ETIMEDOUT, group);
	}

	PTRACE(3, "WinSnd\tTimeout, reopening stuck device");
	if (!OpenDevice(os_handle))
		return false;

	// Throw away signal that is executed in Close() (via OpenDevice())
	if (WaitForSingleObject(hEventDone, 0) == WAIT_OBJECT_0) {
		m_reopened = true;
		return -1;
	}

	PTRACE(2, "WinSnd\tUnexpected error waiting for event, code=" << ::GetLastError());
	}

	return SetErrorValues(Miscellaneous, ::GetLastError()|PWIN32ErrorFlag, group);
}

PBoolean PSoundChannelWin32::Write(const void * data, PINDEX size)
{
	lastWriteCount = 0;

	if (hWaveOut == NULL)
		return SetErrorValues(NotOpen, EBADF, LastWriteError);

	const BYTE * ptr = (const BYTE *)data;

	bufferMutex.Wait();

	DWORD osError = MMSYSERR_NOERROR;
	while (size > 0) {
		PWaveBuffer & buffer = buffers[bufferIndex];
		while ((buffer.header.dwFlags&WHDR_DONE) == 0) {
			bufferMutex.Signal();
			// No free buffers, so wait for one
			switch (WaitEvent(LastWriteError)) {
				case 0 :
					return false;
				case -1 :
					return Write(data, size);
				case 1 :
					break;
			}
			bufferMutex.Wait();
		}

		// Can't write more than a buffer full
		PINDEX count = size;
		if ((osError = buffer.Prepare(hWaveOut, count)) != MMSYSERR_NOERROR)
			break;

		memcpy(buffer.GetPointer(), ptr, count);
		
// rtichange...
#ifdef WAVE_START_DEBUG
		if ( iTraceCount < 10 ) {
			PTRACE(1,"PSoundChannelWin32::Write()- calling waveOutWrite()");
		}
#endif
// ...rtichange
		if ((osError = waveOutWrite(hWaveOut, &buffer.header, sizeof(WAVEHDR))) != MMSYSERR_NOERROR)
			break;
// rtichange...
#ifdef WAVE_START_DEBUG
		// Sleep(100);
		if ( iTraceCount < 10 ) {
			PTRACE(1,"PSoundChannelWin32::Write()- returning from waveOutWrite()");
			iTraceCount++;
		}
#endif
// ...rtichange

		bufferIndex = (bufferIndex+1)%buffers.GetSize();
		lastWriteCount += count;
		size -= count;
		ptr += count;
	}

	if ( delayWriteBuffer ) {
		PTRACE(2,"PSoundChannelWin32::StartRecording() - sleeping kDelayTime milliseconds...");
		Sleep(kDelayTime);
		delayWriteBuffer = false;
	}

	bufferMutex.Signal();

	if (size != 0)
		return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastWriteError);

	return true;
}

PBoolean PSoundChannelWin32::PlaySound(const PSound & sound, PBoolean wait)
{
	Abort();

	PBoolean ok = false;

	PINDEX bufferSize;
	PINDEX bufferCount;
	GetBuffers(bufferSize, bufferCount);

	unsigned numChannels = waveFormat->nChannels;
	unsigned sampleRate = waveFormat->nSamplesPerSec;
	unsigned bitsPerSample = waveFormat->wBitsPerSample;
	if (sound.GetEncoding() == 0)
		ok = SetFormat(sound.GetChannels(), sound.GetSampleRate(), sound.GetSampleSize());
	else {
		waveFormat.SetFormat(sound.GetFormatInfoData(), sound.GetFormatInfoSize());
		ok = OpenDevice(os_handle);
	}

	if (ok) {
		bufferMutex.Wait();

		// To avoid lots of copying of sound data, we fake the PSound buffer into
		// the internal buffers and play directly from the PSound object.
		buffers.SetSize(1);
		PWaveBuffer & buffer = buffers[0];
		buffer = sound;

		DWORD osError;
		PINDEX count = sound.GetSize();

// rtichange...
#ifdef WAVE_START_DEBUG
		if ( iTraceCount < 10 ) {
			PTRACE(1,"PSoundChannelWin32::PlaySound()- calling waveOutWrite()");
		}
#endif
// ...rtichange
		if ((osError = buffer.Prepare(hWaveOut, count)) == MMSYSERR_NOERROR &&
			(osError = waveOutWrite(hWaveOut, &buffer.header, sizeof(WAVEHDR))) == MMSYSERR_NOERROR) {
				if (wait)
					ok = WaitForPlayCompletion();
		}
		else {
			SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastWriteError);
			ok = false;
		}

		bufferMutex.Signal();
	}

	SetFormat(numChannels, sampleRate, bitsPerSample);
	SetBuffers(bufferSize, bufferCount);
	return ok;
}


PBoolean PSoundChannelWin32::HasPlayCompleted()
{
	PWaitAndSignal mutex(bufferMutex);

	for (PINDEX i = 0; i < buffers.GetSize(); i++) {
		if ((buffers[i].header.dwFlags&WHDR_DONE) == 0)
			return false;
	}

	return true;
}

PBoolean PSoundChannelWin32::WaitForPlayCompletion()
{
	while (!HasPlayCompleted()) {
		if (!WaitEvent(LastWriteError))
			return false;
	}

	return true;
}

PBoolean PSoundChannelWin32::StartRecording()
{
	PWaitAndSignal mutex(bufferMutex);

	// See if has started already.
	if (bufferByteOffset != P_MAX_INDEX)
		return true;

	DWORD osError;

	PTRACE(2,"PSoundChannelWin32::StartRecording() - buffers.GetSize() = " << buffers.GetSize() ); // rtichange

	// Start the first read, queue all the buffers
	for (PINDEX i = 0; i < buffers.GetSize(); i++) {
		PWaveBuffer & buffer = buffers[i];
		if ((osError = buffer.Prepare(hWaveIn)) != MMSYSERR_NOERROR)
			return false;
		if ((osError = waveInAddBuffer(hWaveIn, &buffer.header, sizeof(WAVEHDR))) != MMSYSERR_NOERROR)
			return false;
	}

	bufferByteOffset = 0;

	// rtichange...
	// PTRACE(2,"PSoundChannelWin32::StartRecording() - sleeping " << iSleep << " milliseconds...");
	Sleep(kDelayTime); // rtitodo - trying to see if we can get player setup first
	delayWriteBuffer = true;
	PTRACE(2,"PSoundChannelWin32::StartRecording() - calling waveInStart()");
#ifdef WAVE_START_DEBUG
	dwStartTickCount = GetTickCount();
	iTraceCount = 11;
#endif
	// ...rtichange

	if ((osError = waveInStart(hWaveIn)) == MMSYSERR_NOERROR) // start recording
		return true;

	bufferByteOffset = P_MAX_INDEX;
	return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastReadError);
}

PBoolean PSoundChannelWin32::Read(void * data, PINDEX size)
{
	lastReadCount = 0;
	PTRACE(5,"PSoundChannelWin32::Read() - wait til buffer is full..." ); // rtichange

	if (hWaveIn == NULL)
		return SetErrorValues(NotOpen, EBADF, LastReadError);

	if (!WaitForRecordBufferFull())
	{ // rtichange
		PTRACE(2,"PSoundChannelWin32::Read() - !WaitForRecordBufferFull() - return FALSE" ); // rtichange
		return false;
	} // rtichange

// rtichange...
#ifdef WAVE_START_DEBUG
	if ( iTraceCount < 10 ) {
		PTRACE(1,"PSoundChannelWin32::Read() - READ from buffer (WaitForRecordBufferFull() true), " << (GetTickCount() - dwStartTickCount) << "ms" );
		dwStartTickCount = GetTickCount();
	}
	PTRACE(5,"PSoundChannelWin32::Read() - buffer is full, bufferIndex = " << bufferIndex );
#endif
// ...rtichange

	PWaitAndSignal mutex(bufferMutex);

	// Check to see if Abort() was called in another thread
	if (bufferByteOffset == P_MAX_INDEX)
		return false;

	PWaveBuffer & buffer = buffers[bufferIndex];

	lastReadCount = buffer.header.dwBytesRecorded - bufferByteOffset;
	if (lastReadCount > size)
		lastReadCount = size;

	PTRACE(5,"PSoundChannelWin32::Read() - memcpy - lastReadCount = " << lastReadCount ); // rtichange

	memcpy(data, &buffer[bufferByteOffset], lastReadCount);

	bufferByteOffset += lastReadCount;
	if (bufferByteOffset >= (PINDEX)buffer.header.dwBytesRecorded) {
		DWORD osError;
		if ((osError = buffer.Prepare(hWaveIn)) != MMSYSERR_NOERROR)
			return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastReadError);

		// rtichange...
		PTRACE(5,"PSoundChannelWin32::Read() - calling waveInAddBuffer()" );
#ifdef WAVE_START_DEBUG
		if ( iTraceCount < 10 ) {
			PTRACE(1,"PSoundChannelWin32::Read() - call waveInAddBuffer()");
			dwStartTickCount = GetTickCount();
		}
#endif
// ...rtichange

		if ((osError = waveInAddBuffer(hWaveIn, &buffer.header, sizeof(WAVEHDR))) != MMSYSERR_NOERROR)
			return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag, LastReadError);

// rtichange...
#ifdef WAVE_START_DEBUG
		if ( iTraceCount < 10 ) {
			PTRACE(1,"PSoundChannelWin32::Read() - return from waveInAddBuffer()");
			dwStartTickCount = GetTickCount();
			iTraceCount++;
		}
#endif
// ...rtichange

		bufferIndex = (bufferIndex+1)%buffers.GetSize();
		bufferByteOffset = 0;
	}

	return true;
}

PBoolean PSoundChannelWin32::RecordSound(PSound & sound)
{
	if (!WaitForAllRecordBuffersFull())
		return false;

	sound.SetFormat(waveFormat->nChannels,
		waveFormat->nSamplesPerSec,
		waveFormat->wBitsPerSample);

	PWaitAndSignal mutex(bufferMutex);

	if (buffers.GetSize() == 1 &&
		(PINDEX)buffers[0].header.dwBytesRecorded == buffers[0].GetSize())
		sound = buffers[0];
	else {
		PINDEX totalSize = 0;
		PINDEX i;
		for (i = 0; i < buffers.GetSize(); i++)
			totalSize += buffers[i].header.dwBytesRecorded;

		if (!sound.SetSize(totalSize))
			return SetErrorValues(NoMemory, ENOMEM, LastReadError);

		BYTE * ptr = sound.GetPointer();
		for (i = 0; i < buffers.GetSize(); i++) {
			PINDEX sz = buffers[i].header.dwBytesRecorded;
			memcpy(ptr, buffers[i], sz);
			ptr += sz;
		}
	}

	return true;
}

PBoolean PSoundChannelWin32::RecordFile(const PFilePath & filename)
{
	if (!WaitForAllRecordBuffersFull())
		return false;

	PWaitAndSignal mutex(bufferMutex);

	PINDEX dataSize = 0;
	PINDEX i;
	for (i = 0; i < buffers.GetSize(); i++)
		dataSize += buffers[i].header.dwBytesRecorded;

	PMultiMediaFile mmio;
	if (!mmio.CreateWaveFile(filename, waveFormat, dataSize))
		return SetErrorValues(Miscellaneous, mmio.GetLastError()|PWIN32ErrorFlag, LastReadError);

	for (i = 0; i < buffers.GetSize(); i++) {
		if (!mmio.Write(buffers[i], buffers[i].header.dwBytesRecorded))
			return SetErrorValues(Miscellaneous, mmio.GetLastError()|PWIN32ErrorFlag, LastReadError);
	}

	return true;
}

PBoolean PSoundChannelWin32::IsRecordBufferFull()
{
	PWaitAndSignal mutex(bufferMutex);

	return (buffers[bufferIndex].header.dwFlags&WHDR_DONE) != 0 &&
		buffers[bufferIndex].header.dwBytesRecorded > 0;
}

PBoolean PSoundChannelWin32::AreAllRecordBuffersFull()
{
	PWaitAndSignal mutex(bufferMutex);

	for (PINDEX i = 0; i < buffers.GetSize(); i++) {
		if ((buffers[i].header.dwFlags&WHDR_DONE) == 0 ||
			buffers[i].header.dwBytesRecorded    == 0)
			return false;
	}

	return true;
}

PBoolean PSoundChannelWin32::WaitForRecordBufferFull()
{
	if (!StartRecording())  // Start the first read, queue all the buffers
		return false;

	while (!IsRecordBufferFull()) {
		switch (WaitEvent(LastReadError)) {
case 0 :
	return false;

case -1 :
	if (StartRecording())  // Restart the read
		break;
	return false;

case  1 :
	PWaitAndSignal mutex(bufferMutex);
	if (bufferByteOffset == P_MAX_INDEX)
		return false;
		}
	}

	return true;
}

PBoolean PSoundChannelWin32::WaitForAllRecordBuffersFull()
{
	if (!StartRecording())  // Start the first read, queue all the buffers
		return false;

	while (!AreAllRecordBuffersFull()) {
		switch (WaitEvent(LastReadError)) {
case 0 :
	return false;

case -1 :
	if (StartRecording())  // Restart the read
		break;
	return false;

case  1 :
	PWaitAndSignal mutex(bufferMutex);
	if (bufferByteOffset == P_MAX_INDEX)
		return false;
		}
	}

	return true;
}

PBoolean PSoundChannelWin32::Abort()
{
	DWORD osError = MMSYSERR_NOERROR;

	{
		PWaitAndSignal mutex(bufferMutex);

		if (hWaveOut != NULL || hWaveIn != NULL) {
			for (PINDEX i = 0; i < buffers.GetSize(); i++) {
				do {
					if (hWaveOut != NULL)
						waveOutReset(hWaveOut);
					if (hWaveIn != NULL)
						waveInReset(hWaveIn);
				} while (buffers[i].Release() == WAVERR_STILLPLAYING);
			}
		}

		bufferByteOffset = P_MAX_INDEX;
		bufferIndex = 0;

		// Signal any threads waiting on this event, they should then check
		// the bufferByteOffset variable for an abort.
		SetEvent(hEventDone);
	}

	if (osError != MMSYSERR_NOERROR)
		return SetErrorValues(Miscellaneous, osError|PWIN32ErrorFlag);

	return true;
}

PString PSoundChannelWin32::GetErrorText(ErrorGroup group) const
{
	char str[256];

	if ((lastErrorNumber[group]&PWIN32ErrorFlag) == 0)
		return PChannel::GetErrorText(group);

	DWORD osError = lastErrorNumber[group]&~PWIN32ErrorFlag;
	if (activeDirection == Recorder) {
		if (waveInGetErrorText(osError, str, sizeof(str)) != MMSYSERR_NOERROR)
			return PChannel::GetErrorText(group);
	}
	else {
		if (waveOutGetErrorText(osError, str, sizeof(str)) != MMSYSERR_NOERROR)
			return PChannel::GetErrorText(group);
	}

	return str;
}

PBoolean PSoundChannelWin32::SetVolume(unsigned newVolume)
{
	if (!IsOpen() || hMixer == NULL)
		return SetErrorValues(NotOpen, EBADF);
	
	// this can be mic volume! for rti, volume is adjusted via app callback // rtichange
	PTRACE(2,"PSoundChannelWin32::SetVolume()"); // rtichange

	// unsigned int oldVolume = 0; // rtichange
	// GetVolume(oldVolume); // rtichange
	// PTRACE(1,"PSoundChannelWin32::SetVolume() - old volume = " << oldVolume ); // rtichange

	MIXERCONTROLDETAILS_UNSIGNED volume;
	if (newVolume >= MaxVolume) // rtichange rtitodo
		volume.dwValue = volumeControl.Bounds.dwMaximum; // rtichange
	else // rtichange
		volume.dwValue = volumeControl.Bounds.dwMinimum + (DWORD)((volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum)*newVolume/MaxVolume);
	PTRACE(2, "WinSnd\tVolume set to " << newVolume << " -> " << volume.dwValue); // rtichange

	MIXERCONTROLDETAILS details;
	details.cbStruct = sizeof(details);
	details.dwControlID = volumeControl.dwControlID;
	details.cChannels = 1;
	details.cMultipleItems = 0;
	details.cbDetails = sizeof(volume);
	details.paDetails = &volume;

	MMRESULT result = mixerSetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE);
	if (result != MMSYSERR_NOERROR)
		return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);

	return true;
}

PBoolean PSoundChannelWin32::GetVolume(unsigned & oldVolume)
{
	if (!IsOpen() || hMixer == NULL)
		return SetErrorValues(NotOpen, EBADF);

	MIXERCONTROLDETAILS_UNSIGNED volume;

	MIXERCONTROLDETAILS details;
	details.cbStruct = sizeof(details);
	details.dwControlID = volumeControl.dwControlID;
	details.cChannels = 1;
	details.cMultipleItems = 0;
	details.cbDetails = sizeof(volume);
	details.paDetails = &volume;

	MMRESULT result = mixerGetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE);
	if (result != MMSYSERR_NOERROR)
		return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);

	oldVolume = MaxVolume*(volume.dwValue + 1 - volumeControl.Bounds.dwMinimum)/(volumeControl.Bounds.dwMaximum - volumeControl.Bounds.dwMinimum);
	return true;
}

bool PSoundChannelWin32::SetMute(bool newMute)
{
	if (!IsOpen() || hMixer == NULL)
		return SetErrorValues(NotOpen, EBADF);

	MIXERCONTROLDETAILS_UNSIGNED mute;
	mute.dwValue = newMute;
	PTRACE(5, "WinSnd\tMute set to " << newMute << " -> " << mute.dwValue);

	MIXERCONTROLDETAILS details;
	details.cbStruct = sizeof(details);
	details.dwControlID = muteControl.dwControlID;
	details.cChannels = 1;
	details.cMultipleItems = 0;
	details.cbDetails = sizeof(mute);
	details.paDetails = &mute;

	MMRESULT result = mixerSetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_SETCONTROLDETAILSF_VALUE);
	if (result != MMSYSERR_NOERROR)
		return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);

	return true;
}

bool PSoundChannelWin32::GetMute(bool & oldMute)
{
	if (!IsOpen() || hMixer == NULL)
		return SetErrorValues(NotOpen, EBADF);

	MIXERCONTROLDETAILS_UNSIGNED mute;

	MIXERCONTROLDETAILS details;
	details.cbStruct = sizeof(details);
	details.dwControlID = muteControl.dwControlID;
	details.cChannels = 1;
	details.cMultipleItems = 0;
	details.cbDetails = sizeof(mute);
	details.paDetails = &mute;

	MMRESULT result = mixerGetControlDetails((HMIXEROBJ)hMixer, &details, MIXER_OBJECTF_HMIXER | MIXER_GETCONTROLDETAILSF_VALUE);
	if (result != MMSYSERR_NOERROR)
		return SetErrorValues(Miscellaneous, result|PWIN32ErrorFlag);

	oldMute = mute.dwValue;
	return true;
}


// End of File ///////////////////////////////////////////////////////////////



// rtichange...
#ifdef _WIN32_WCE

// -----------------------------------------------------------------------------
//                              FileHeader
// -----------------------------------------------------------------------------
typedef struct
{
	DWORD   dwRiff;     // Type of file header.
	DWORD   dwSize;     // Size of file header.
	DWORD   dwWave;     // Type of wave.
} RIFF_FILEHEADER, *PRIFF_FILEHEADER;


// -----------------------------------------------------------------------------
//                              ChunkHeader
// -----------------------------------------------------------------------------
typedef struct
{
	DWORD   dwCKID;        // Type Identification for current chunk header.
	DWORD   dwSize;        // Size of current chunk header.
} RIFF_CHUNKHEADER, *PRIFF_CHUNKHEADER;

/*  Chunk Types  
*/
#define RIFF_FILE       mmioFOURCC('R','I','F','F')
#define RIFF_WAVE       mmioFOURCC('W','A','V','E')
#define RIFF_FORMAT     mmioFOURCC('f','m','t',' ')
#define RIFF_CHANNEL    mmioFOURCC('d','a','t','a')

BOOL
ReadChunk(HANDLE fh, DWORD dwChunkType, PVOID * ppBuffer, DWORD * pdwSize, PDWORD pdwBytesLeft)
{ DWORD dwBytesRead;
PVOID pBuffer;
RIFF_CHUNKHEADER Chunk;
Chunk.dwSize = 0;	// satisfy compiler warning

if ((!pdwBytesLeft) || (*pdwBytesLeft <= 0) || (!pdwSize) || (!ppBuffer)) {
	RETAILMSG(1, (TEXT("Invalid parameter to ReadChunk()\r\n")));
	return FALSE;
}

// now scan for the format chunk
while (*pdwBytesLeft > 0) {
	// now read the wave header (or what we hope is the wave header)
	if (! ReadFile(fh, &Chunk, sizeof(Chunk), &dwBytesRead, NULL) || dwBytesRead < sizeof(Chunk)) {
		RETAILMSG(1, (TEXT("Error reading chunk header\n")));
		return FALSE;
	}
	*pdwBytesLeft -= dwBytesRead;
	RETAILMSG(1, (TEXT("Chunk: \"%c%c%c%c\" size=0x%08x\r\n"), 
		(Chunk.dwCKID >>  0) & 0xff, 
		(Chunk.dwCKID >>  8) & 0xff, 
		(Chunk.dwCKID >> 16) & 0xff, 
		(Chunk.dwCKID >> 24) & 0xff, 
		Chunk.dwSize));
	if (Chunk.dwCKID == dwChunkType) {
		// found the desired chunk
		break;
	}
	// skip the data we don't know or care about...
	if (0xFFFFFFFF == SetFilePointer (fh, Chunk.dwSize, NULL, FILE_CURRENT)) {
		RETAILMSG(1,  (TEXT("Error setting file pointer while scanning for chunk\n")));
		return FALSE;
	}
	*pdwBytesLeft -= Chunk.dwSize;
}
// found the desired chunk.
// allocate a buffer and read in the data
pBuffer = new BYTE[Chunk.dwSize];
if (pBuffer == NULL) {
	RETAILMSG(1, (TEXT("Unable to allocate chunk buffer\r\n")));
	return FALSE;
}
if (! ReadFile(fh, pBuffer, Chunk.dwSize, &dwBytesRead, NULL) || dwBytesRead < Chunk.dwSize) {
	delete [] pBuffer;
	RETAILMSG(1, (TEXT("Unable to read chunk data\r\n")));
	return FALSE;
}
*pdwBytesLeft -= dwBytesRead;
*ppBuffer = pBuffer;
*pdwSize = Chunk.dwSize;
return TRUE;
}

MMRESULT
ReadWaveFile(LPCTSTR pszFilename, PWAVEFORMATEX * ppWFX, PDWORD pdwBufferSize, PBYTE * ppBufferBits)
{ RIFF_FILEHEADER FileHeader;
DWORD dwBytesRead;
DWORD dwBufferSize;
DWORD dwFormatSize;
PBYTE pBufferBits = NULL;
PWAVEFORMATEX pwfx = NULL;
DWORD dwBytesInChunk;
HANDLE fh;
MMRESULT mmRet = MMSYSERR_ERROR;

fh = CreateFile(pszFilename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
if( fh == INVALID_HANDLE_VALUE ) {
	RETAILMSG(1, (TEXT("Error opening %s. Error code = 0x%08x\n"), pszFilename, GetLastError()));
	return mmRet;
}

// Read file and determine sound format
// Start with RIFF header:

if (! ReadFile(fh, &FileHeader, sizeof(FileHeader), &dwBytesRead, NULL) || dwBytesRead < sizeof(FileHeader)) {
	RETAILMSG(1, (TEXT("Error reading file header\n")));
	goto ERROR_EXIT;
}

if ( FileHeader.dwRiff != RIFF_FILE || FileHeader.dwWave != RIFF_WAVE) {
	RETAILMSG(1, (TEXT("Invalid wave file header\n")));
	goto ERROR_EXIT;
}

dwBytesInChunk = FileHeader.dwSize;

// load the wave format
if (! ReadChunk(fh, RIFF_FORMAT, (PVOID*) &pwfx, &dwFormatSize, &dwBytesInChunk)) {
	RETAILMSG(1, (TEXT("Unable to read format chunk\r\n")));
	goto ERROR_EXIT;
}
if (dwFormatSize < sizeof(PCMWAVEFORMAT)) {
	RETAILMSG(1, (TEXT("Format record too small\r\n")));
	goto ERROR_EXIT;
}

// load the wave data
if (! ReadChunk(fh, RIFF_CHANNEL, (PVOID*) &pBufferBits, &dwBufferSize, &dwBytesInChunk)) {
	RETAILMSG(1, (TEXT("Unable to read format chunk\r\n")));
	goto ERROR_EXIT;
}

*ppWFX = pwfx;
*pdwBufferSize = dwBufferSize;
*ppBufferBits = pBufferBits;

// Success
mmRet = MMSYSERR_NOERROR;
goto EXIT;

ERROR_EXIT:
delete [] pBufferBits;
delete [] pwfx;
EXIT:
CloseHandle(fh);
return mmRet;   
}



MMRESULT
WriteWaveFile (LPCTSTR pszFilename, PWAVEFORMATEX pWFX, DWORD dwBufferSize, PBYTE pBufferBits)
{ RIFF_FILEHEADER FileHeader;
RIFF_CHUNKHEADER WaveHeader;
RIFF_CHUNKHEADER DataHeader;
DWORD dwBytesWritten;
HANDLE fh;
MMRESULT mmRet = MMSYSERR_ERROR;


// Fill in the file, wave and data headers
WaveHeader.dwCKID = RIFF_FORMAT;
WaveHeader.dwSize = sizeof(WAVEFORMATEX) + pWFX->cbSize;

// the DataHeader chunk contains the audio data
DataHeader.dwCKID = RIFF_CHANNEL;
DataHeader.dwSize = dwBufferSize;

// The FileHeader
FileHeader.dwRiff = RIFF_FILE;
FileHeader.dwWave = RIFF_WAVE;
FileHeader.dwSize = sizeof(FileHeader.dwWave)+sizeof(WaveHeader) + WaveHeader.dwSize + sizeof(DataHeader) + DataHeader.dwSize;

// Open wave file
fh = CreateFile(pszFilename, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
if( fh == INVALID_HANDLE_VALUE ) {
	RETAILMSG(1, (TEXT("Error opening %s. Error code = 0x%08x\n"), pszFilename, GetLastError()));
	return mmRet;
}

// write the riff file
if (! WriteFile(fh, &FileHeader, sizeof(FileHeader), &dwBytesWritten, NULL)) {
	RETAILMSG(1, (TEXT("Error writing file header\r\n")));
	goto ERROR_EXIT;
}

// write the wave header
if (! WriteFile(fh, &WaveHeader, sizeof(WaveHeader), &dwBytesWritten, NULL)) {
	RETAILMSG(1, (TEXT("Error writing wave header\r\n")));
	goto ERROR_EXIT;
}

// write the wave format
if (! WriteFile(fh, pWFX, WaveHeader.dwSize, &dwBytesWritten, NULL)) {
	RETAILMSG(1, (TEXT("Error writing wave format\r\n")));
	goto ERROR_EXIT;
}

// write the data header
if (! WriteFile(fh, &DataHeader, sizeof(DataHeader), &dwBytesWritten, NULL)) {
	RETAILMSG(1, (TEXT("Error writing PCM data header\r\n")));
	goto ERROR_EXIT;
}

// write the PCM data
if (! WriteFile(fh, pBufferBits, DataHeader.dwSize, &dwBytesWritten, NULL)) {
	RETAILMSG(1, (TEXT("Error writing PCM data\r\n")));
	goto ERROR_EXIT;
}

// Success
mmRet = MMSYSERR_NOERROR;

ERROR_EXIT:
CloseHandle(fh);
return mmRet;
}


MMRESULT
RecordWaveBuffer (PWAVEHDR pwh, DWORD dwDeviceId, PWAVEFORMATEX pwfx, DWORD dwDuration)
{ const DWORD dwTolerance = 1000;
  DWORD dwWait;
  HANDLE hevDone;
  HWAVEIN hwi;

    // create an event so we know when capture is completed
    hevDone = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (hevDone == NULL) {
        RETAILMSG (1, (TEXT("Unable to create completion event\r\n")));
        return MMSYSERR_NOMEM;
    }

    // open the wave capture device
    MMRESULT mr = waveInOpen(&hwi, dwDeviceId, pwfx, (DWORD) hevDone, NULL, CALLBACK_EVENT);
    if (mr != MMSYSERR_NOERROR) {
        RETAILMSG(1, (TEXT("waveInOpen failed. mr=%08x\r\n"), mr));
        return mr;
    }

    // set up the WAVEHDR structure that describes the capture buffer

    // prepare the buffer for capture
    mr = waveInPrepareHeader(hwi, pwh, sizeof(WAVEHDR));

    // submit the buffer for capture
    mr = waveInAddBuffer(hwi, pwh, sizeof(WAVEHDR));

    // start capturing
    RETAILMSG(1, (TEXT("Starting capture...\r\n")));
    mr = waveInStart(hwi);
	// bRecording = TRUE;

    // wait for completion + 1 second tolerance
    dwWait= WaitForSingleObject(hevDone, dwDuration + dwTolerance);
	// bRecording = FALSE;
    if (dwWait != WAIT_OBJECT_0) {
        RETAILMSG(1, (TEXT("Timeout waiting for capture to complete, writing partialy file.\r\n")));
        mr = waveInReset(hwi);
        if (mr != MMSYSERR_NOERROR) {
            RETAILMSG(1, (TEXT("warning: waveInReset failed. mr=%08x\r\n"), mr));
        }
    }

    // now clean up: unprepare the buffer
    mr = waveInUnprepareHeader(hwi, pwh, sizeof(WAVEHDR));

// rtitodo ERROR_DONE:
    // close the capture device & free the event handle
    mr = waveInClose(hwi);
    if (mr != MMSYSERR_NOERROR) {
        RETAILMSG(1, (TEXT("warning: waveInClose failed. mr=%08x\r\n"), mr));
    }

    CloseHandle(hevDone);

    return mr;
}

MMRESULT
RecordWaveFile (PCTSTR pszFilename, DWORD dwDeviceId, PWAVEFORMATEX pwfx, DWORD dwDuration)
{ MMRESULT mr;
  PBYTE pBufferBits;
  DWORD dwBufferSize;

    // compute the size of the capture buffer
    dwBufferSize = (DWORD)((((UINT64)dwDuration) * ((UINT64)pwfx->nAvgBytesPerSec) / 1000));

    // Pad out to a multiple of nBlockAlign
    dwBufferSize += dwBufferSize % pwfx->nBlockAlign;

    // let user know what's going on
    RETAILMSG(1, (TEXT("Recording %5d ms to \"%s\": %c%02d %5dHz (%8d bytes)\r\n")
        , dwDuration
        , pszFilename
        , pwfx->nChannels == 2 ? L'S' : L'M'
        , pwfx->wBitsPerSample
        , pwfx->nSamplesPerSec
        , dwBufferSize
        ));

    // try to allocate the capture buffer
    pBufferBits = new BYTE [dwBufferSize];
    if (pBufferBits == NULL) {
        RETAILMSG (1, (TEXT("Unable to allocate %d bytes for %d ms of audio data\r\n"), dwBufferSize, dwDuration));
        return MMSYSERR_NOMEM;
    }
    WAVEHDR hdr;
    memset(&hdr, 0, sizeof(WAVEHDR));
    hdr.dwBufferLength = dwBufferSize;
    hdr.lpData = (char *) pBufferBits;

    mr = RecordWaveBuffer(&hdr, dwDeviceId, pwfx, dwDuration);

    // finally, write the captured buffer to the file
    // note that we use hdr.dwBytesRecorded, not dwBuffersize.
    RETAILMSG(1, (TEXT("Capture completed. Writing %s\r\n"), pszFilename));
    mr = WriteWaveFile(pszFilename, pwfx, hdr.dwBytesRecorded, pBufferBits);

// rtitodo ERROR_DONE:
    // free up the capture buffer and release the done event
    delete [] pBufferBits;

    // and we're done
    return mr;
}


// =========================================================================================
// WaveFile.cpp - from kx7 HW diags

MMRESULT RecordWaveFileHuh (PCTSTR pszFilename, DWORD dwDeviceId, PWAVEFORMATEX pwfx, DWORD dwDuration)
{
	MMRESULT mr;
	PBYTE pBufferBits1;
	PBYTE pBufferBits2;
	DWORD dwBufferSize;
	const DWORD dwTolerance = 1000;
	DWORD dwWait;
	HANDLE hevDone;
	HANDLE hevDoneOut;
	HWAVEIN hwi;
	HWAVEOUT hwo;
	WAVEHDR hdr1;
	WAVEHDR hdr2;

	// compute the size of the capture buffer
	dwBufferSize = (DWORD)((((UINT64)dwDuration) * ((UINT64)pwfx->nAvgBytesPerSec) / 1000));

	// Pad out to a multiple of nBlockAlign
	dwBufferSize += dwBufferSize % pwfx->nBlockAlign;

	// rtitodo dwBufferSize /= 10;
	dwBufferSize = 160;

	// let user know what's going on
	RETAILMSG(1, (TEXT("Recording %5d ms to \"%s\": %c%02d %5dHz (%8d bytes)\r\n"), dwDuration, pszFilename, pwfx->nChannels == 2 ? L'S' : L'M', pwfx->wBitsPerSample, pwfx->nSamplesPerSec, dwBufferSize));

	// create an event so we know when capture is completed
	hevDone = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (hevDone == NULL) {
		RETAILMSG (1, (TEXT("Unable to create completion event\r\n")));
		return MMSYSERR_NOMEM;
	}

	// open the wave capture device
	mr = waveInOpen(&hwi, dwDeviceId, pwfx, (DWORD) hevDone, NULL, CALLBACK_EVENT);
	if (mr != MMSYSERR_NOERROR) {
		RETAILMSG(1, (TEXT("waveInOpen failed. mr=%08x\r\n"), mr));
		return mr;
	}


	// allocate two buffers

	// 1st buffer...
	pBufferBits1 = new BYTE [dwBufferSize*2];

	memset(&hdr1, 0, sizeof(WAVEHDR));
	hdr1.dwBufferLength = dwBufferSize;
	hdr1.lpData = (char *) pBufferBits1;

	// prepare the buffer for capture
	mr = waveInPrepareHeader(hwi, &hdr1, sizeof(WAVEHDR));

	// submit the buffer for capture
	mr = waveInAddBuffer(hwi, &hdr1, sizeof(WAVEHDR));


	// 2nd buffer...
	pBufferBits2 = new BYTE [dwBufferSize*2];

	memset(&hdr2, 0, sizeof(WAVEHDR));
	hdr2.dwBufferLength = dwBufferSize;
	hdr2.lpData = (char *) pBufferBits2;

	// prepare the buffer for capture
	mr = waveInPrepareHeader(hwi, &hdr2, sizeof(WAVEHDR));

	// submit the buffer for capture
	mr = waveInAddBuffer(hwi, &hdr2, sizeof(WAVEHDR));


	WAVEFORMATEX wfx;
	wfx.cbSize				= 0;
	wfx.wFormatTag			= WAVE_FORMAT_PCM;
	wfx.wBitsPerSample		= (WORD) 16;
	wfx.nSamplesPerSec		= 8000;
	wfx.nChannels			= (WORD) 1;
	wfx.nBlockAlign			= wfx.nChannels * wfx.wBitsPerSample / 8;
	wfx.nAvgBytesPerSec		= wfx.nBlockAlign * wfx.nSamplesPerSec;

	// create an event so we know when capture is completed
	hevDoneOut = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (hevDoneOut == NULL) {
		RETAILMSG (1, (TEXT("Unable to create completion event\r\n")));
		return MMSYSERR_NOMEM;
	}

	waveOutOpen(&hwo, (UINT)WAVE_MAPPER, &wfx, (DWORD_PTR)hevDoneOut, 0, CALLBACK_EVENT);



	// start capturing
	RETAILMSG(1, (TEXT("Starting capture...\r\n")));
	mr = waveInStart(hwi);


	// so my assumption is that the rx buffer full issue is caused by a delay when trying to read the buffers.  If we 
	// our buffers fill up before we have a chance to read from them, then those errors occur.  The question is, when
	// does the waveIn code decide when to stop filling the buffers?  Or doesn't it (which is really the problem).  If
	// it doesn't stop filling the buffers, and we don't give it more buffers soon enough, then we should see the rx 
	// buffer overflow errors.  So we should be able to add a long delay here and see that happen, correct?  Or should
	// the delay be after the wait?  Does it matter?

	Sleep(20000);

	// wait for completion + 1 second tolerance
	dwWait= WaitForSingleObject(hevDone, dwDuration + dwTolerance);

	if (dwWait != WAIT_OBJECT_0) {
		RETAILMSG(1, (TEXT("Timeout waiting for capture to complete, writing partialy file.\r\n")));
		mr = waveInReset(hwi);
		if (mr != MMSYSERR_NOERROR) {
			RETAILMSG(1, (TEXT("warning: waveInReset failed. mr=%08x\r\n"), mr));
		}
	}

	// now clean up: unprepare the buffer
	mr = waveInUnprepareHeader(hwi, &hdr1, sizeof(WAVEHDR));
	mr = waveInUnprepareHeader(hwi, &hdr2, sizeof(WAVEHDR));

	// close the capture device & free the event handle
	mr = waveInClose(hwi);
	if (mr != MMSYSERR_NOERROR) {
		RETAILMSG(1, (TEXT("warning: waveInClose failed. mr=%08x\r\n"), mr));
	}

	CloseHandle(hevDone);

	// finally, write the captured buffer to the file
	// note that we use hdr.dwBytesRecorded, not dwBuffersize.
	RETAILMSG(1, (TEXT("Capture completed. Writing %s\r\n"), pszFilename));
	mr = WriteWaveFile(pszFilename, pwfx, hdr1.dwBytesRecorded, pBufferBits1);

	// free up the capture buffer and release the done event
	delete [] pBufferBits1;
	delete [] pBufferBits2;

	// and we're done
	return mr;
}



MMRESULT RecordWaveFileBad(PCTSTR pszFilename, DWORD dwDeviceId, PWAVEFORMATEX pwfx, DWORD dwDuration)
{
	MMRESULT mr;
	PBYTE pBufferBits1;
	// PBYTE pBufferBits2;
	// PBYTE pBufferBits3;
	DWORD dwBufferSize;
	const DWORD dwTolerance = 1000;
	DWORD dwWait;
	HANDLE hevDone;
	// HANDLE hevDoneOut;
	HWAVEIN hwi;
	// HWAVEOUT hwo;
	WAVEHDR hdr1;
	// WAVEHDR hdr2;
	// WAVEHDR hdrOut;


	/* these are the trace messages in the fail case...
	1. waveInOpen() is implied here, not in traces
	2. PID:057700AA TID:0683000E 2015/05/06 15:02:45.000         Event Pool:109248526  sound_win32.cxx(797)   PSoundChannelWin32::OpenDevice() - Player! - waveOutOpen() returned NOERROR
	3. PID:057700AA TID:06910012 2015/05/06 15:02:45.000         Media Patch:110166034  sound_win32.cxx(1506)  PSoundChannelWin32::StartRecording() - calling waveInStart()
	4. PID:057700AA TID:06980012 2015/05/06 15:02:45.000         Media Patch:110624786  sound_win32.cxx(1349)  PSoundChannelWin32::Write()- calling waveOutWrite()
	5. PID:057700AA TID:06910012 2015/05/06 15:02:45.000         Media Patch:110166034  sound_win32.cxx(1532)  PSoundChannelWin32::Read() - READ from buffer (WaitForRecordBufferFull() true), 70ms
	6. PID:057700AA TID:06980012 2015/05/06 15:02:45.000         Media Patch:110624786  sound_win32.cxx(1354)  PSoundChannelWin32::Write()- returning from waveOutWrite()
	7. PID:00400002 TID:01D4004E ERROR: c:\wince700\platform\mx35\src\drivers\wavedev\audiodev\record\recctxt.cpp line 845: PID:00400002 TID:01D4004E ERROR: SSI1 RX overrun (x1000)error
	8. PID:057700AA TID:06980012 2015/05/06 15:02:45.000		 Media Patch:110624786  sound_win32.cxx(1349)  PSoundChannelWin32::Write()- calling waveOutWrite()
	9. PID:057700AA TID:06980012 2015/05/06 15:02:45.000         Media Patch:110624786  sound_win32.cxx(1354)  PSoundChannelWin32::Write()- returning from waveOutWrite()
	A. PID:057700AA TID:06980012 2015/05/06 15:02:45.000         Media Patch:110624786  sound_win32.cxx(1349)  PSoundChannelWin32::Write()- calling waveOutWrite()
	B. PID:00400002 TID:01D4004E ERROR: c:\wince700\platform\mx35\src\drivers\wavedev\audiodev\record\recctxt.cpp line 845: PID:00400002 TID:01D4004E ERROR: SSI1 RX overrun (x1000)error
	C. PID:00400002 TID:01D4004E ERROR: c:\wince700\platform\mx35\src\drivers\wavedev\audiodev\record\recctxt.cpp line 845: PID:00400002 TID:01D4004E ERROR: SSI1 RX overrun (x1000)error
	D. PID:057700AA TID:06910012 2015/05/06 15:02:45.000         Media Patch:110166034  sound_win32.cxx(1563)  PSoundChannelWin32::Read() - call waveInAddBuffer()
	*/


	//
	// 1.  waveInOpen() is implied here, not in traces
	//

	// compute the size of the capture buffer
	dwBufferSize = (DWORD)((((UINT64)dwDuration) * ((UINT64)pwfx->nAvgBytesPerSec) / 1000));

	// Pad out to a multiple of nBlockAlign
	dwBufferSize += dwBufferSize % pwfx->nBlockAlign;

	// rtitodo dwBufferSize /= 10;
	// dwBufferSize = 160;

	// let user know what's going on
	RETAILMSG(1, (TEXT("Recording %5d ms to \"%s\": %c%02d %5dHz (%8d bytes)\r\n"), dwDuration, pszFilename, pwfx->nChannels == 2 ? L'S' : L'M', pwfx->wBitsPerSample, pwfx->nSamplesPerSec, dwBufferSize));

	// create an event so we know when capture is completed
	hevDone = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (hevDone == NULL) {
		RETAILMSG (1, (TEXT("Unable to create completion event\r\n")));
		return MMSYSERR_NOMEM;
	}

	// open the wave capture device
	mr = waveInOpen(&hwi, dwDeviceId, pwfx, (DWORD) hevDone, NULL, CALLBACK_EVENT);
	if (mr != MMSYSERR_NOERROR) {
		RETAILMSG(1, (TEXT("waveInOpen failed. mr=%08x\r\n"), mr));
		return mr;
	}


	DWORD dwStart =  GetTickCount();



	/*
	//
	// 2. PID:057700AA TID:0683000E 2015/05/06 15:02:45.000         Event Pool:109248526  sound_win32.cxx(797)   PSoundChannelWin32::OpenDevice() - Player! - waveOutOpen() returned NOERROR
	//
	WAVEFORMATEX wfx;
	wfx.cbSize				= 0;
	wfx.wFormatTag			= WAVE_FORMAT_PCM;
	wfx.wBitsPerSample		= (WORD) 16;
	wfx.nSamplesPerSec		= 8000;
	wfx.nChannels			= (WORD) 1;
	wfx.nBlockAlign			= wfx.nChannels * wfx.wBitsPerSample / 8;
	wfx.nAvgBytesPerSec		= wfx.nBlockAlign * wfx.nSamplesPerSec;

	// create an event so we know when capture is completed
	hevDoneOut = CreateEvent(NULL, FALSE, FALSE, NULL);
	if (hevDoneOut == NULL) {
		RETAILMSG (1, (TEXT("Unable to create completion event\r\n")));
		return MMSYSERR_NOMEM;
	}


	RETAILMSG(1, (TEXT("CALLING waveOutOpen()\r\n")));
	waveOutOpen(&hwo, (UINT)WAVE_MAPPER, &wfx, (DWORD_PTR)hevDoneOut, 0, CALLBACK_EVENT);

	DWORD dwCurrent =  GetTickCount();
	RETAILMSG(1, (TEXT("RecordWaveFileBad() - waveOutOpen() time = %ld\r\n"), dwCurrent - dwStart));
	dwStart = GetTickCount();
	*/


	//
	// 3. PID:057700AA TID:06910012 2015/05/06 15:02:45.000         Media Patch:110166034  sound_win32.cxx(1506)  PSoundChannelWin32::StartRecording() - calling waveInStart()
	//

	// allocate two buffers

	// 1st buffer...
	pBufferBits1 = new BYTE [dwBufferSize*2];

	memset(&hdr1, 0, sizeof(WAVEHDR));
	hdr1.dwBufferLength = dwBufferSize;
	hdr1.lpData = (char *) pBufferBits1;

	// prepare the buffer for capture
	mr = waveInPrepareHeader(hwi, &hdr1, sizeof(WAVEHDR));

	// submit the buffer for capture
	RETAILMSG(1, (TEXT("CALLING waveInAddBuffer() 1\r\n")));
	mr = waveInAddBuffer(hwi, &hdr1, sizeof(WAVEHDR));


	// 2nd buffer...
	/*
	pBufferBits2 = new BYTE [dwBufferSize*2];

	memset(&hdr2, 0, sizeof(WAVEHDR));
	hdr2.dwBufferLength = dwBufferSize;
	hdr2.lpData = (char *) pBufferBits2;

	// prepare the buffer for capture
	mr = waveInPrepareHeader(hwi, &hdr2, sizeof(WAVEHDR));

	// submit the buffer for capture
	RETAILMSG(1, (TEXT("CALLING waveInAddBuffer() 2\r\n")));
	mr = waveInAddBuffer(hwi, &hdr2, sizeof(WAVEHDR)); */





	// 4. PID:057700AA TID:06980012 2015/05/06 15:02:45.000         Media Patch:110624786  sound_win32.cxx(1349)  PSoundChannelWin32::Write()- calling waveOutWrite()

	// 2nd buffer...
	// pBufferBits3 = new BYTE [dwBufferSize*2];

	/*
	memset(&hdrOut, 0, sizeof(WAVEHDR));
	hdrOut.dwBufferLength = dwBufferSize;
	hdrOut.lpData = (char *) pBufferBits3;

	// prepare the buffer for capture
	waveOutPrepareHeader(hwo, &hdrOut, sizeof(WAVEHDR));

	RETAILMSG(1, (TEXT("CALLING waveOutWrite()\r\n")));
	waveOutWrite(hwo, &hdrOut, sizeof(WAVEHDR));

	dwCurrent =  GetTickCount();
	RETAILMSG(1, (TEXT("RecordWaveFileBad() - waveOutWrite() time = %ld\r\n"), dwCurrent - dwStart));
	dwStart = GetTickCount(); */






	// start capturing
	RETAILMSG(1, (TEXT("CALLING waveInStart()\r\n")));
	mr = waveInStart(hwi);

	// dwCurrent =  GetTickCount();
	// RETAILMSG(1, (TEXT("RecordWaveFileBad() - waveIn add buffers and start time = %ld\r\n"), dwCurrent - dwStart));
	dwStart = GetTickCount();






	//
	// 7. PID:00400002 TID:01D4004E ERROR: c:\wince700\platform\mx35\src\drivers\wavedev\audiodev\record\recctxt.cpp line 845: PID:00400002 TID:01D4004E ERROR: SSI1 RX overrun (x1000)error
	//



	// 
	// D. PID:057700AA TID:06910012 2015/05/06 15:02:45.000         Media Patch:110166034  sound_win32.cxx(1563)  PSoundChannelWin32::Read() - call waveInAddBuffer()
	//

	// wait for completion + 1 second tolerance
	dwWait= WaitForSingleObject(hevDone, dwDuration + dwTolerance);

	if (dwWait != WAIT_OBJECT_0) {
		RETAILMSG(1, (TEXT("Timeout waiting for capture to complete, writing partialy file.\r\n")));

		RETAILMSG(1, (TEXT("CALLING waveInReset()\r\n")));
		mr = waveInReset(hwi);
		if (mr != MMSYSERR_NOERROR) {
			RETAILMSG(1, (TEXT("warning: waveInReset failed. mr=%08x\r\n"), mr));
		}
	}

	// now clean up: unprepare the buffer
	RETAILMSG(1, (TEXT("CALLING waveInUnprepareHeader()\r\n")));
	mr = waveInUnprepareHeader(hwi, &hdr1, sizeof(WAVEHDR));

	// RETAILMSG(1, (TEXT("CALLING waveInUnprepareHeader()\r\n")));
	// mr = waveInUnprepareHeader(hwi, &hdr2, sizeof(WAVEHDR));

	// close the capture device & free the event handle
	RETAILMSG(1, (TEXT("CALLING waveInClose()\r\n")));
	mr = waveInClose(hwi);
	if (mr != MMSYSERR_NOERROR) {
		RETAILMSG(1, (TEXT("warning: waveInClose failed. mr=%08x\r\n"), mr));
	}

	CloseHandle(hevDone);



	// close the capture device & free the event handle
	/*
	RETAILMSG(1, (TEXT("CALLING waveOutClose()\r\n")));
	mr = waveOutClose(hwo);
	CloseHandle(hevDoneOut); */



	// finally, write the captured buffer to the file
	// note that we use hdr.dwBytesRecorded, not dwBuffersize.
	// RETAILMSG(1, (TEXT("Capture completed. Writing %s\r\n"), pszFilename));
	// mr = WriteWaveFile(pszFilename, pwfx, hdr1.dwBytesRecorded, pBufferBits1);

	// free up the capture buffer and release the done event
	delete [] pBufferBits1;
	// delete [] pBufferBits2;
	// delete [] pBufferBits3;

	// and we're done
	return mr;
}

int RecordMic()
{ 
    // The run parameters
    // Set up defaults to be overridden by command line
    PTSTR pszFilename = TEXT("windows\\wav.wav");
	//PTSTR pszFilename = TEXT("IPSM\\wav.wav");
    DWORD dwDuration = 10 * 1000;    // record for 5 seconds
    DWORD dwChannels = 1;           // default to mono
    DWORD dwBitsPerSample = 16;     // default to 8-bit samples
    DWORD dwSampleRate = 8000; //changed to match PMIC   11025;     // default to 11.025KHz sample rate
    DWORD dwDeviceId = WAVE_MAPPER;           // capture from any available device  

    // set up the wave format structure
#if 0
    // Turn this on to test GSM610 ACM codec
    GSM610WAVEFORMAT wfx;
    wfx.wfx.wFormatTag = WAVE_FORMAT_GSM610;
    wfx.wfx.nChannels = 0x0001;
    wfx.wfx.nSamplesPerSec = 0x1f40;
    wfx.wfx.nAvgBytesPerSec = 0x00000659;
    wfx.wfx.nBlockAlign = 0x0041;
    wfx.wfx.wBitsPerSample = 0x0000;
    wfx.wfx.cbSize = 0x0002;
    wfx.wSamplesPerBlock = 0x140;
#else    
    WAVEFORMATEX wfx;
    wfx.cbSize = 0;
    wfx.wFormatTag = WAVE_FORMAT_PCM;
    wfx.wBitsPerSample = (WORD) dwBitsPerSample;
    wfx.nSamplesPerSec = dwSampleRate;
    wfx.nChannels = (WORD) dwChannels;
    wfx.nBlockAlign = wfx.nChannels * wfx.wBitsPerSample / 8;
    wfx.nAvgBytesPerSec = wfx.nBlockAlign * wfx.nSamplesPerSec;
#endif

    // we're all set: go do the real work
    RecordWaveFile(pszFilename, dwDeviceId, (PWAVEFORMATEX)&wfx, dwDuration);

    return 0;
}

int RecordMicErrors()
{ 
#ifdef _WIN32_WCE
	PTSTR pszFilename = TEXT("\\ipsm\\wav.wav");
#else
	PTSTR pszFilename = TEXT("C:\\Temp\\wav.wav");
#endif

	WAVEFORMATEX wfx;
	wfx.cbSize				= 0;
	wfx.wFormatTag			= WAVE_FORMAT_PCM;
	wfx.wBitsPerSample		= (WORD) 8; // 16;
	wfx.nSamplesPerSec		= 8000;
	wfx.nChannels			= (WORD) 1;
	wfx.nBlockAlign			= wfx.nChannels * wfx.wBitsPerSample / 8;
	wfx.nAvgBytesPerSec		= wfx.nBlockAlign * wfx.nSamplesPerSec;

	// we're all set: go do the real work
	// RecordWaveFileBad(pszFilename, WAVE_MAPPER, (PWAVEFORMATEX)&wfx, 5000);
	RecordWaveFile(pszFilename, WAVE_MAPPER, (PWAVEFORMATEX)&wfx, 5000);

	return 0;
}

#endif
// ...rtichange